module Config.Pages.Debate.Gibberish.Helpers exposing (..)

import Config.Helpers.CardFormat exposing (..)
import Config.Helpers.Format
    exposing
        ( paragraphFontSize
        , paragraphSpacing
        )
import Config.Helpers.StrengthBar
    exposing
        ( barMaker
        , barPadding
        )
import Config.Helpers.ToolTip exposing (tooltip)
import Config.Pages.Debate.Arguments.Types exposing (..)
import Config.Pages.Debate.Gibberish.Domains.Epistemology
import Config.Pages.Debate.Gibberish.Domains.Metaphysics
import Config.Pages.Debate.Gibberish.Domains.Normativity
import Config.Pages.Debate.Gibberish.Domains.Ontology
import Config.Pages.Debate.Gibberish.Domains.PhilOfLanguage
import Config.Pages.Debate.Gibberish.Domains.PhilOfMind
import Config.Pages.Debate.Gibberish.Domains.Theology
import Config.Pages.Debate.Gibberish.Types exposing (..)
import Config.Style.Colour exposing (colourTheme)
import Element as E exposing (..)
import Element.Background as B
import Element.Border as D
import Element.Font as F


desktopGibberishMaker : Gibberish -> Element msg
desktopGibberishMaker gibberish =
    row
        topLevelBox
        [ desktopCardMaker desktopImageBoxSize desktopImageSize (gibberishImage gibberish) gibberish.gibberishLink
        , cardMaker
            [ cardTitleMaker (gibberishTitle gibberish)
            , cardFormatter
                [ column [ spacing 20, padding 10 ]
                    [ descriptionMaker gibberish
                    , gibberishMakerBody gibberish
                    ]
                ]
            ]
        ]


mobileGibberishMaker : Gibberish -> Element msg
mobileGibberishMaker gibberish =
    row
        topLevelBox
        [ cardMaker
            [ cardTitleMaker (gibberishTitle gibberish)
            , cardFormatter
                [ row [ spacing 20, padding 10 ]
                    [ column [] [ mobileCardMaker mobileImageBoxSize mobileImageSize (gibberishImage gibberish) gibberish.gibberishLink ]
                    , column [ width fill ]
                        [ descriptionMaker gibberish
                        ]
                    ]
                , domainList gibberish
                ]
            ]
        ]


gibberishMakerBody : Gibberish -> Element msg
gibberishMakerBody gibberish =
    cardContentSpacing
        [ column
            fieldSpacer
            [ paragraph
                [ F.color colourTheme.textLightGrey
                , paragraphSpacing
                , paragraphFontSize
                , F.size 18
                , F.center
                ]
                [ domainList gibberish
                ]
            ]
        ]


gibberishImage : Gibberish -> { src : String, description : String }
gibberishImage gibberish =
    { src = "/gibberish/" ++ gibberish.gibberishImage ++ ".png"
    , description = gibberish.gibberishTitle
    }


gibberishTitle : Gibberish -> String
gibberishTitle gibberish =
    gibberish.gibberishTitle


domainList : Gibberish -> Element msg
domainList gibberish =
    column
        [ spacing 8
        , E.width fill
        ]
        (List.indexedMap (makeTerms intelligibilityBar) gibberish.gibberishTerms)


descriptionMaker : Gibberish -> Element msg
descriptionMaker gibberish =
    column
        [ E.width fill
        , centerX
        ]
        [ paragraph
            [ F.color colourTheme.textLightGrey
            , paragraphSpacing
            , paragraphFontSize
            , spacing 3
            , F.regular
            , F.alignLeft
            , F.size 16
            ]
            [ text gibberish.gibberishDescription
            ]
        ]


makeTerms : (Terms -> Element msg) -> Int -> Terms -> Element msg
makeTerms bar index terms =
    column
        [ F.color colourTheme.textLightGrey
        , paragraphSpacing
        , paragraphFontSize
        , F.bold
        , E.alignLeft
        , spacing 8
        , E.width fill
        ]
        [ row
            [ F.color colourTheme.textLightGrey
            , F.regular
            , F.size 18
            , F.bold
            , F.alignLeft
            , E.width fill
            ]
            [ column
                [ E.alignRight
                , alignTop
                ]
                [ paragraph
                    []
                    [ text (String.fromInt (index + 1) ++ ". ") ]
                ]
            , column
                [ E.width fill
                , paddingEach
                    { top = 0
                    , bottom = 0
                    , left = 10
                    , right = 10
                    }
                , spacing 3
                ]
                [ paragraph
                    []
                    [ el [ F.color colourTheme.textLightOrange ] <|
                        text <|
                            String.toLower "\""
                                ++ terms.term
                                ++ "\""
                    ]
                , bar terms
                , paragraph
                    [ F.color colourTheme.textLightGrey
                    , F.regular
                    , F.size 16
                    , F.alignLeft
                    ]
                    [ text
                        (case terms.explanation of
                            NoClue ->
                                "I have no fucking clue what this means."

                            SpecificExplanation str ->
                                str
                        )
                    ]
                ]
            ]
        ]


intelligibilityBar : Terms -> Element msg
intelligibilityBar terms =
    row
        [ E.width fill
        , height fill
        ]
        [ column
            [ E.alignTop
            , E.alignLeft
            ]
            [ paragraph
                [ F.color colourTheme.textLightGrey
                , F.bold
                , F.size 18
                , paragraphSpacing
                , paragraphFontSize
                , E.alignLeft
                , E.width fill
                ]
                [ el
                    [ tooltip
                        tooltipMessage
                    ]
                    (el
                        [ paddingEach
                            { top = 0
                            , right = 10
                            , bottom = 0
                            , left = 0
                            }
                        ]
                     <|
                        text barTitle
                    )
                ]
            ]
        , barPadding
            [ barMaker getIntelligibilityTooltip terms.strength ]
        ]


tooltipMessage : String
tooltipMessage =
    "This represents my confidence that the term can be understood from at least one viewpoint."


barTitle : String
barTitle =
    "Intelligibility:"


getIntelligibilityTooltip : Int -> String
getIntelligibilityTooltip num =
    case num of
        0 ->
            "Total fucking gibberish."

        1 ->
            "Extremely unclear, speaking in tongues."

        2 ->
            "Mostly unclear, hard to make any sense of."

        3 ->
            "Somewhat unclear, difficult to understand."

        4 ->
            "Slightly unclear, understandable with effort."

        5 ->
            "Neutral, not sure what to make of it."

        6 ->
            "Slightly clear, with unanswered questions."

        7 ->
            "Somewhat clear, kinda get the idea."

        8 ->
            "Very clear, usefulness questionable."

        9 ->
            "Extremely clear, usefulness dubious."

        10 ->
            "Perfectly clear, but also useless."

        _ ->
            "Intelligibility rating is out of bounds."


gibberishListNumber : Int
gibberishListNumber =
    List.length Config.Pages.Debate.Gibberish.Domains.Epistemology.epistemologyGibberish.gibberishTerms
        + List.length Config.Pages.Debate.Gibberish.Domains.Metaphysics.metaphysicsGibberish.gibberishTerms
        + List.length Config.Pages.Debate.Gibberish.Domains.Normativity.normativityGibberish.gibberishTerms
        + List.length Config.Pages.Debate.Gibberish.Domains.Ontology.ontologyGibberish.gibberishTerms
        + List.length Config.Pages.Debate.Gibberish.Domains.PhilOfLanguage.philOfLanguageGibberish.gibberishTerms
        + List.length Config.Pages.Debate.Gibberish.Domains.PhilOfMind.philOfMindGibberish.gibberishTerms
        + List.length Config.Pages.Debate.Gibberish.Domains.Theology.theologyGibberish.gibberishTerms
